/**
 * Created by Evalds on 13/06/2017.
 */

const fs = require('fs');
const path = require('path');
const _ = require('lodash');
const os = require('os');

const logger = require('./Logger');
const DevelopmentTypes = require('./DevelopmentTypes');
const RequestMessageTypes = require('../common/RequestMessageTypes');

const XLSX = require('xlsx');
const SystemUtils = require('./SystemUtils');
const RegistryUtils = require('./RegistryUtils');

module.exports = class FileUtils {

    static getDataStoragePath(isElectron)
    {
        let regUtils = false;
        if(isElectron) {
            const {remote} = require('electron');
            regUtils = remote.getGlobal('getRegistryUtilsElectron')();
        } else {
            let WindowsRegistry = null;
            if(os.platform() === 'win32') {
                WindowsRegistry = require('windows-registry');
            }
            regUtils = new RegistryUtils(WindowsRegistry);
        }

        let pathDataStorage = regUtils.getDataStoragePath();
        if(pathDataStorage) {
            if (!fs.existsSync(pathDataStorage)) {
                fs.mkdirSync(pathDataStorage);
            }
            return pathDataStorage;
        }

        return path.join(SystemUtils.getAppDataPath(), 'ChromSword');
    }

    static getAppDataPath()
    {
        return SystemUtils.getAppDataPath();
    }

    // Get file extensions (for legacy support of ChromSword 5.0)
    static getExtensionByDevelopmentType(dev_type_id) {
        var ext = 'prjx';
        try
        {
            switch(dev_type_id)
            {
                case DevelopmentTypes.ReportViewer:
                    ext = 'rptx';
                    break;
                case DevelopmentTypes.DataSystem:
                    ext = 'prjx';
                    break;
                case DevelopmentTypes.Scout:
                    ext = 'prjs';
                    break;
                case DevelopmentTypes.Developer:
                    ext = 'prjd';
                    break;
                case DevelopmentTypes.AutoRobust:
                    ext = 'prja';
                    break;
                case DevelopmentTypes.PumpTest:
                    ext = 'prjp';
                    break;
            }
        }
        catch (exc)
        {
            logger.error('FileUtils.getExtensionByDevelopmentType');
            logger.error(exc);
        }
        return ext;
    }

    // Save file from request
    static saveFile(id, ext, folder, fileBase64, options) {

        let _options = _.defaults(options || {}, {
            localPath : false,
            isElectron: false
        });

        try
        {
            let filepath = '';

            if(_options.localPath === false){

                filepath = path.join(FileUtils.getDataStoragePath(_options.isElectron), 'ClientServer', folder, id.toString() + '.' + ext);
                if( !fs.existsSync(path.join(FileUtils.getDataStoragePath(_options.isElectron), 'ClientServer', folder)) ){
                    fs.mkdirSync(path.join(FileUtils.getDataStoragePath(_options.isElectron), 'ClientServer', folder));
                }
            }
            else{
                filepath = _options.localPath;
            }


            let buffer = Buffer.from(fileBase64, 'base64');

            fs.writeFileSync(filepath, buffer,  "binary");

        }
        catch (exc)
        {
            logger.error('FileUtils.saveFile');
            logger.error(exc);
        }
    }

    static downloadFile(id, ext, folder, options){
        let _options = _.defaults(options || {}, {
            isElectron: false
        });
        let filedata = '';
        let empty_file = false;
        try{
            var filepath = path.join(FileUtils.getDataStoragePath(_options.isElectron), 'ClientServer', folder, id.toString() + '.' + ext);
            if(fs.existsSync(filepath)){
                filedata = fs.readFileSync(filepath, 'binary');
            }
            else{
                empty_file = true;
            }
        }
        catch(exc){
            logger.error('FileUtils.downloadFile');
            logger.error(exc);
        }
        if(empty_file){
            return false;
        }
        return new Buffer(filedata, 'binary').toString('base64');
    }

    static createExportViewFile(data){
        let wopts = { bookType:'xlsx', bookSST:false, type:'binary' };
        let wb = XLSX.utils.book_new();
        let ws = XLSX.utils.aoa_to_sheet(data);
        XLSX.utils.book_append_sheet(wb, ws, 'ChromSword');
        let wbout = XLSX.write(wb, wopts);
        return new Buffer(wbout, 'binary').toString('base64');
    }
}
