var winston = require('winston');
require('winston-daily-rotate-file');

const fs = require('fs');
const path = require('path');
const SystemUtils = require('../common/SystemUtils');

// export ENV=development
var level = process.env.ENV === 'development' ? 'debug' : 'info';

var dir_parts = __dirname.split(path.sep);
var dir_app_name = dir_parts[dir_parts.length - 3]; // common, app|release, name

var dir_logging = path.join(SystemUtils.getAppDataPath(), 'ChromSword');
if (!fs.existsSync(dir_logging)) {
    fs.mkdirSync(dir_logging);
}

dir_logging = path.join(SystemUtils.getAppDataPath(), 'ChromSword', 'Logs');
if (!fs.existsSync(dir_logging)) {
    fs.mkdirSync(dir_logging);
}


var transport = new winston.transports.DailyRotateFile({
    filename: '-' + dir_app_name + '.log',
    dirname: dir_logging,
    datePattern: 'yyyy-MM-dd',
    prepend: true,
    level: level,
    json: false
});

let loggerWinston = new (winston.Logger)({
    transports: [
        new (winston.transports.Console)({
            level: level
        }),
        transport
    ]
});

const MAX_LIMIT_LOG_ENTRY = 1000;
module.exports = class Logger {
      static formatOutput(...params) {
          let result = [];
          try {
              for(let obj of params) {
                  if (typeof obj !== "object") {
                      result.push(obj.toString());
                  }
                  else {
                      let str_result = JSON.stringify(obj, null, 4);
                      if (str_result.length > MAX_LIMIT_LOG_ENTRY) {
                          str_result = str_result.substr(0, MAX_LIMIT_LOG_ENTRY);
                          str_result += '... [truncated]';
                      }
                      result.push(str_result);
                  }
              }
          }
          catch(exc){
              if (process.env.ENV === 'development') {
                  console.error(exc);
              }
          }
          if(result.length === 0) {
              result.push('n/a');
          }
          return result.join('\n');
      }

    static info(...obj) {
        try {
            loggerWinston.info(this.formatOutput.apply(null, obj));
        }
        catch(exc){
            if (process.env.ENV === 'development') {
                console.error(exc);
            }
        }
    }

    static debug(...obj) {
        try {
            loggerWinston.debug(this.formatOutput.apply(null, obj));
        }
        catch(exc){
            if (process.env.ENV === 'development') {
                console.error(exc);
            }
        }
    }


    static error(...obj) {
        try {
            let isLogged = false;
            if(obj.length === 1) {
                if(obj[0] instanceof Error) {
                    isLogged = true;
                    loggerWinston.error(obj[0].message);
                    if (process.env.ENV === 'development') {
                        if(obj[0].line) {
                            loggerWinston.error(`line: ${obj[0].line}`);
                        }
                        loggerWinston.error(obj[0].stack);
                    }
                }
            }
            if(!isLogged) {
                loggerWinston.error(this.formatOutput.apply(null, obj));
            }
        }
        catch(exc){
            if (process.env.ENV === 'development') {
                console.error(exc);
            }
        }
    }

    static warn(...obj) {
        try {
            loggerWinston.warn(this.formatOutput.apply(null, obj));
        }
        catch(exc){
            if (process.env.ENV === 'development') {
                console.error(exc);
            }
        }
    }

};