// FDA password requirements get from here: https://www.fda.gov/Food/GuidanceRegulation/FoodFacilityRegistration/ucm091108.htm

module.exports = class PasswordHandler{
    // Return false if OK
    static testPassStrength(pass){
        if(process.env.CS_WEAK_PASSWORD)
        {
            return false;
        }

        if(pass.length < 8 || pass.length >= 32){ // At least 8, but no more than 32 characters.
            return true;
        }
        if(/\s+/.test(pass)){ // space char not allowed
            return true;
        }
        if(!/[A-Z]{1,}/.test(pass)){ // At least one UPPERCASE letter.
            return true;
        }
        if(!/[a-z]{1,}/.test(pass)){ // At least one lowercase letter.
            return true;
        }
        if(!/[\~\!\@\#\$\%\^\*\(\)\_\+\=\{\}\[\]\|\:\;\"\,\?\-]{1,}/.test(pass)){ // At least one special character: ~ ! @ # $ % ^ * ( ) _ - + = { } [ ] | : ; " , ?. Do not use < >& or '.
            return true;
        }
        if(!/[0-9]{1,}/.test(pass)){ // At least one number.
            return true;
        }
        return false;
    }

    static passExpireCheck(password_date, password_time, settings){
        if(password_date && password_time) {
            let timeDiff = Math.abs((new Date()).getTime() - new Date(password_date.toISOString().slice(0, 10) + ' ' + password_time).getTime());
            let diffDays = timeDiff / (1000 * 3600 * 24);

            if (diffDays >= settings.password_expiration_time) {
                return true;
            }
        }
        return false;
    }
}
