/**
 * Created by Evalds on 06/07/2017.
 */


// https://github.com/CatalystCode/windows-registry-node
//const WindowsRegistry = require('windows-registry');

const CHROMSWORD_KEY_REG = 'SOFTWARE\\Chromsword\\Auto';
const CHROMSWORD_KEY_REG_32 = 'SOFTWARE\\WOW6432Node\\Chromsword\\Auto';
const CHROMSWORD_VALUE_ADDRESS = 'ClientServerAddress';
const CHROMSWORD_VALUE_STORAGE_PATH = 'ClientServerStoragePath';
const CHROMSWORD_VALUE_REPORT_SERVICE_TIMEOUT = 'ClientServerReportServiceTimeout';
const CHROMSWORD_LANGUAGE = 'Language';

module.exports = class RegistryUtils {
    constructor(windowsRegistry) {
        this._windowsRegistry = windowsRegistry;
    }

    getValue64_or_32bit(name_value_in_reg_key) {
        let value = false;
        console.log('name_value_in_reg_key', name_value_in_reg_key);
        try {
            for(let hive of [
                this._windowsRegistry.windef.HKEY.HKEY_CURRENT_USER,
                this._windowsRegistry.windef.HKEY.HKEY_LOCAL_MACHINE
            ]) {
                try {
                    let key = new this._windowsRegistry.Key(
                        hive,
                        CHROMSWORD_KEY_REG_32,
                        this._windowsRegistry.windef.KEY_ACCESS.KEY_READ);

                    if (key) {
                        try {
                            value = key.getValue(name_value_in_reg_key);
                            console.log('success', value, CHROMSWORD_KEY_REG_32, hive);
                            key.close();
                        } catch (e) {}
                    }
                }
                catch(e) {}

                try {
                    if (!value) {
                        let key = new this._windowsRegistry.Key(
                            hive,
                            CHROMSWORD_KEY_REG,
                            this._windowsRegistry.windef.KEY_ACCESS.KEY_READ);

                        if (key) {
                            try {
                                value = key.getValue(name_value_in_reg_key);
                                console.log('success', value, CHROMSWORD_KEY_REG, hive);
                                key.close();
                            } catch (e) {
                            }
                        }
                    }
                }
                catch (e) {}

                if(value) {
                    break;
                }
            }
        }
        catch(exc) {
            console.error(exc);
        }
        return value;
    }

    writeValue64_or_32bit(name_value_in_reg_key, value) {
        try {
            let base_key = new this._windowsRegistry.Key(
                this._windowsRegistry.windef.HKEY.HKEY_CURRENT_USER,
                "SOFTWARE\\WOW6432Node",
                this._windowsRegistry.windef.KEY_ACCESS.KEY_ALL_ACCESS);
            try {
                let sub_key = null;
                try{
                    sub_key = base_key.openSubKey("Chromsword", this._windowsRegistry.windef.KEY_ACCESS.KEY_ALL_ACCESS);
                }
                catch (e) {
                    sub_key = base_key.createSubKey("Chromsword", this._windowsRegistry.windef.KEY_ACCESS.KEY_ALL_ACCESS);
                }
                let sub_sub_key = null;
                try{
                    sub_sub_key = base_key.openSubKey("Chromsword\\Auto", this._windowsRegistry.windef.KEY_ACCESS.KEY_ALL_ACCESS);
                }
                catch (e) {
                    sub_sub_key = base_key.createSubKey("Chromsword\\Auto", this._windowsRegistry.windef.KEY_ACCESS.KEY_ALL_ACCESS);
                }
            } catch(e) {
                console.log(e)
            }
            console.log('debug-4-1');
            let key = new this._windowsRegistry.Key(
                this._windowsRegistry.windef.HKEY.HKEY_CURRENT_USER,
                CHROMSWORD_KEY_REG_32,
                this._windowsRegistry.windef.KEY_ACCESS.KEY_ALL_ACCESS);
            console.log('debug-4');

            key.setValue(name_value_in_reg_key, this._windowsRegistry.windef.REG_VALUE_TYPE.REG_SZ, value);
            key.close();
        }
        catch(exc) {
            console.error(exc);
        }
    }

    getClientServerAddress() {
        let value = false;

        try {
            value = this.getValue64_or_32bit(CHROMSWORD_VALUE_ADDRESS);
            if(value.indexOf('localhost') >= 0) {
                value = value.replace('localhost', '127.0.0.1');
            }
        }
        catch(exc) {
            console.log(exc);
        }

        return value;
    }

    getReportServiceTimeout() {
        let value = 5000;

        try {
            value = this.getValue64_or_32bit(CHROMSWORD_VALUE_REPORT_SERVICE_TIMEOUT);
        }
        catch(exc) {
            console.log(exc);
        }

        return value;
    }

    getDataStoragePath() {
        let value = false;

        try {
            value = this.getValue64_or_32bit(CHROMSWORD_VALUE_STORAGE_PATH);
        }
        catch(exc) {
            console.log(exc);
        }

        return value;
    }

    getLanguage() {
        let value = false;

        try {
            value = this.getValue64_or_32bit(CHROMSWORD_LANGUAGE);
            value = value.toLowerCase();
        }
        catch(exc) {
            console.log(exc);
        }

        return value;
    }

    setLanguage(lang) {

        try {
            this.writeValue64_or_32bit(CHROMSWORD_LANGUAGE, lang);
        }
        catch(exc) {
            console.log(exc);
        }
    }

};