/**
 * Created by Evalds on 01/06/2017.
 */

const logger = require('./Logger');
const FileUtils = require('./FileUtils');
const service = require ("os-service"); // https://www.npmjs.com/package/os-service
const fs = require('fs');
const path = require('path');
const shell = require('shelljs'); // https://www.npmjs.com/package/shelljs
const _ = require('lodash');

module.exports = class WindowsService {

    constructor(serviceName, readableServiceName)
    {
        this._serviceName = serviceName;
        this._readableServiceName = readableServiceName;
    }

    async register() {
        try {
            await this.remove();
            await this.add();
            //Start service
            shell.exec("net start \"" + this._serviceName + "\"");
        }
        catch(exc) {
            logger.error(exc);
        }
    }

    async add() {
        var basePath = __dirname + '/../../';
        var self = this;


        return new Promise((resolve, reject) => {
                service.add(self._serviceName, {
                    programPath: basePath + 'release/app.js',
                    programArgs: ['--run'],
                    displayName: self._readableServiceName
                }, function (error) {
                    if (error)
                        logger.error(error);

                    logger.info('WindowsService.add');
                    resolve();
                });
            });
    }

    async remove() {
        var self = this;

        return new Promise((resolve, reject) => {
            shell.exec("net stop \"" + self._serviceName + "\"", function(code, stdout, stderr) {
                service.remove(self._serviceName, function (error) {
                    if (error)
                        logger.error(error);

                    logger.info('WindowsService.remove');
                    resolve();
                });
            });
        });
    }

    async run() {
        var self = this;
        return new Promise((resolve, reject) => {

            logger.info('WindowsService.run start');
            try
            {
                var dir_logging = path.join(FileUtils.getAppDataPath(), 'ChromSword');
                if (!fs.existsSync(dir_logging)) {
                    fs.mkdirSync(dir_logging);
                }

                dir_logging = path.join(FileUtils.getAppDataPath(), 'ChromSword', 'Logs');
                if (!fs.existsSync(dir_logging)) {
                    fs.mkdirSync(dir_logging);
                }

                var date = new Date();

                self.logStream = fs.createWriteStream (path.join(dir_logging,
                    date.getFullYear().toString() + '-' +
                    _.padStart((date.getMonth() + 1).toString(), 2, '0') + '-' +
                    _.padStart(date.getDate().toString(), 2, '0') + '-' +
                    'stdout-' + self._serviceName + '.log'));

                service.run (self.logStream, function () {
                    logger.info('WindowsService.service.run');

                    //Stop request
                    service.stop (0);
                });

                logger.info('WindowsService.run started');
            }
            catch (exc) {
                logger.error('WindowsService.run');
                logger.error(exc);
            }


            resolve();
        });
    }
}

