/**
 * Created by Evalds on 24/05/2017.
 */


const express = require('express');
const bodyParser = require('body-parser');

const UsersController = require('../controllers/UsersController');
const RolesController = require('../controllers/RolesController');
const AuditTrailController = require('../controllers/AuditTrailController');
const InstrumentsController = require('../controllers/InstrumentsController');
const ProjectGroupsController = require('../controllers/ProjectGroupsController');
const SessionTokensController = require('../controllers/SessionTokensController');
const ProjectController = require('../controllers/ProjectController');
const UsersRolesController = require('../controllers/UsersRolesController');
const ReportController = require('../controllers/ReportController');
const SettingsController = require('../controllers/SettingsController');
const RequestsController = require('../controllers/RequestsController');

const i18n = require('../utils/i18n');
const pgAsync = require('../utils/Database');
const CodeUtils = require('../utils/CodeUtils');
const Responses = require('../utils/Responses');
const Requests = require('../utils/Requests');

const SQL = require('pg-async').SQL;

const DateTimeUtils = require('../common/DateTimeUtils');
const EnumUtils = require('../common/EnumUtils');

const crypto = require('crypto');

const fs = require('fs');
const path = require('path');

const _ = require('lodash');

const logger = require('../common/Logger');
const FileUtils = require('../common/FileUtils');
const CryptoUtils = require('../common/CryptoUtils');
const EventTypes = require('../common/EventTypes');
const DevelopmentTypes = require('../common/DevelopmentTypes');
const VendorTypes = require('../common/VendorTypes');
const RequestMessageTypes = require('../common/RequestMessageTypes');
const PasswordHandler = require('../common/PasswordHandler');
const RegistryUtils = require('../common/RegistryUtils');

const pjson = require('../../package.json');

const util = require('util')

const os = require('os');
const LoggerUtils = require("../common/LoggerUtils");
// Windows specific code
let ref = null;
let ffi = null;
let WindowsRegistry = null;
if(os.platform() === 'win32') {
    ref = require("ref");
    ffi = require('ffi');
    WindowsRegistry = require('windows-registry');
}

const {promisify} = require('util');
const mv = promisify(fs.rename);

/*
 Main entry point for application
 */

const CLIENT_SERVER_PORT = 8888; //default port if not overridden

module.exports = class ClientServer {

    constructor() {
        try {
            logger.info('ChromSword ClientServer started', pjson.version);

            this.app = express();
            this.app.use(bodyParser.json({limit: '1000mb', type: 'application/json'}));

            let registry_utils = new RegistryUtils(WindowsRegistry);
            let lang = registry_utils.getLanguage();
            i18n.init(lang);

            //Create file structures
            var dir = path.join(FileUtils.getAppDataPath(), 'ChromSword');
            if (!fs.existsSync(dir)) {
                fs.mkdirSync(dir);
            }
            dir = path.join(FileUtils.getDataStoragePath());
            if (!fs.existsSync(dir)) {
                fs.mkdirSync(dir);
            }
            dir = path.join(FileUtils.getDataStoragePath(), 'ClientServer');
            if (!fs.existsSync(dir)) {
                fs.mkdirSync(dir);
            }
            dir = path.join(FileUtils.getDataStoragePath(), 'ClientServer', 'Projects');
            if (!fs.existsSync(dir)) {
                fs.mkdirSync(dir);
            }
            dir = path.join(FileUtils.getDataStoragePath(), 'ClientServer', 'Reports');
            if (!fs.existsSync(dir)) {
                fs.mkdirSync(dir);
            }

            this.validate();

            //Migrate existing data to data storage path from legacy dir
            for(let strPart of ['Reports', 'Projects']) {
                let dirLegacy = path.join(FileUtils.getAppDataPath(), 'ChromSword', 'ClientServer', strPart);
                if (fs.existsSync(dirLegacy)) {
                    logger.info(`legacy dir exists: ${dirLegacy}`);
                    fs.readdir(dirLegacy, function (err, files) {
                        try {
                            if (!err) {
                                for (let file of files) {
                                    mv(
                                        path.join(dirLegacy, file),
                                        path.join(FileUtils.getDataStoragePath(), 'ClientServer', strPart, file));
                                    logger.info(`moving: ${file}`);
                                }
                            }
                        }
                        catch (exc) {
                            logger.error(exc);
                        }
                    });
                }
            }

            this.app.post('*', (httpRequest, httpResponse) => {

                try {
                    this.process(httpRequest.body, httpResponse).catch((err) => {
                        logger.error(err);
                        httpResponse.send(Responses.Default);
                    });
                }
                catch (exc) {
                    logger.error(exc);
                    httpResponse.send(Responses.Default);
                }
            });

            // Read port from CS installation registry
            let port = CLIENT_SERVER_PORT;
            if(os.platform() === 'win32') {
                let clientServerAddress = registry_utils.getClientServerAddress();
                if (clientServerAddress) {
                    port = _.last(clientServerAddress.split(':'))
                }
                logger.info('listening', port, clientServerAddress);
            }
            this.app.listen(port);
        }
        catch (exc)
        {
            logger.error(exc);
        }
    }

    validate () {

        //Cannot do validation because this is 64bit app
        //Validation of HASP key, DLL is encrypted
        // if (os.platform() === 'win32') {
        //     try {
        //         var tokenTimestamp = "";
        //         for (var i = 0; i < 13; i++) {
        //             tokenTimestamp += "0";
        //         }
        //
        //         //Only x86 arch
        //         var libPath = "X5uUINW5P70lqyj54rcjqX83Wb3MDZRE8mttNV3Fa+I+VfE52CJ32w/Yc1nvLbqr";
        //
        //         //libPath = CryptoUtils.cipherToken("\\..\\libraries\\ChromSword.ClientServer", tokenTimestamp);
        //         //console.log(libPath);
        //         libPath = CryptoUtils.decipherToken(libPath, tokenTimestamp);
        //
        //         var libValidate = ffi.Library(__dirname + libPath, {
        //             "Validate": ['string', ['string']]
        //         });
        //
        //         //Salt ClientServer
        //         var salt = "fenJWIRYDCNApBop7uXDsQ==";
        //         //var salt = CryptoUtils.cipherToken("901", tokenTimestamp);
        //         //console.log(salt);
        //
        //         var hashInput = CryptoUtils.md5((Math.random() * 10e9).toString());
        //         var hashOutput = libValidate.Validate(hashInput);
        //         //console.log(hashOutput);
        //         if (hashOutput !== CryptoUtils.md5(hashInput + CryptoUtils.decipherToken(salt, tokenTimestamp))) {
        //             return false;
        //         }
        //     }
        //     catch(exc)
        //     {
        //         logger.error(exc);
        //     }
        // }

        return true;
    }

    async process(request, httpResponse) {

        if(process.env.ENV === 'development') {
            logger.debug(`Request: ${EnumUtils.Stringify(request.messageType, RequestMessageTypes)}`);
        }

        let response = Responses.Default;
        let project = undefined;
        let report = undefined;
        let settings = {
            password_expiration_time: 15,
            max_user_session_time: 30
        };
        try {
            let settingsOverride = await SettingsController.getAll();
            settings = {...settings, ...settingsOverride};
        }
        catch (exc) {
            logger.error(exc);
        }

        try {

            let devTypeName = _.invert(DevelopmentTypes);
            let vendorTypeName = _.invert(VendorTypes);
            let eventTypeName = _.invert(EventTypes);

            if (!('timestamp' in request)) {
                logger.error('request timestamp missing');
            }
            if (!('checksum' in request)) {
                logger.error('request checksum missing');
            }

            // important to delete file and checksum before calculating checksum for validation
            let checksum = request.checksum;
            delete request.checksum;
            let file = request.file;
            if (file) {
                delete request.file;
            } else {
                file = false;
            }

            if (checksum !== CryptoUtils.md5('chk' + JSON.stringify(request) + request.timestamp.toString())) {
                logger.error('malicious request');
                if(process.env.ENV === 'development') {
                    LoggerUtils.logRequest(request);
                }
                response.messageType = Requests.MessageTypes.InvalidChecksum;
                response.success = false;
            } else {
                if(file) {
                    request.file = file;
                }
                if(await RequestsController.isRequestAlreadyRegistered(request)) {
                    logger.warn('repeated request');
                    if(process.env.ENV === 'development') {
                        LoggerUtils.logRequest(request);
                    }
                    response.messageType = Requests.MessageTypes.InvalidRepeatedRequest;
                } else {
                    await RequestsController.registerRequest(request);
                    let sessionToken = request.sessionToken ? request.sessionToken : SessionTokensController.Default;

                    try {

                        if (sessionToken.is_valid) {
                            sessionToken = await SessionTokensController.getByHash(sessionToken.hash);

                            if ((new Date()).getTime() - sessionToken.last_used.getTime() >= settings.max_user_session_time * 60 * 1000 &&
                                request.messageType === Requests.MessageTypes.Logout) {
                                sessionToken.is_valid = false;
                            }

                            await SessionTokensController.update(sessionToken);
                            sessionToken.password_expired = PasswordHandler.passExpireCheck(sessionToken.user.password_date, sessionToken.user.password_time, settings);
                        }


                        // Validate using HASP key
                        if (!this.validate()) {
                            sessionToken.is_valid = false;
                        }

                        let auditTrail = AuditTrailController.Default;
                        auditTrail.event_type_id = AuditTrailController.EventTypes.Info;
                        auditTrail.dev_type_id = AuditTrailController.DevelopmentTypes.AdminConsole;

                        //Avoid sending huge data for database serialization
                        //Files can be sent also with invalid session token

                        if (sessionToken.is_valid) {

                            let user = {};
                            let serverFolder = '';
                            let item2download = {};
                            let file_type = '';
                            let file_ext = false;

                            switch (request.messageType) {

                                case Requests.MessageTypes.None:
                                    response.success = true;
                                    break;

                                case Requests.MessageTypes.AdminViewSettings:
                                    response.settings = await SettingsController.getAll();
                                    response.success = true;
                                    break;

                                case Requests.MessageTypes.AdminEditSettings:
                                    response.success = await SettingsController.saveAll(request.settings);
                                    break;

                                case Requests.MessageTypes.AdminViewOverview:
                                    let selectedProjectGroups = [];
                                    if (request.selectedProjectGroups.length > 0) {
                                        for (let req_pgs of request.selectedProjectGroups) {
                                            selectedProjectGroups.push({id: req_pgs});
                                        }
                                    }
                                    else {
                                        selectedProjectGroups = await ProjectGroupsController.getAvailableProjects(sessionToken);
                                    }
                                    response.data_dev_type = await ReportController.getProgramUsageOverview(selectedProjectGroups, request.dateFrom, request.dateTo, sessionToken);
                                    response.data_users_values = await ReportController.getActiveUsersOverview(selectedProjectGroups, request.dateFrom, request.dateTo, sessionToken);
                                    response.data_instruments = await ReportController.getInstrumentsUsageOverview(selectedProjectGroups, request.dateFrom, request.dateTo, sessionToken);
                                    response.data_runs = await ReportController.getRunsOverview(selectedProjectGroups, request.dateFrom, request.dateTo, sessionToken);
                                    response.data_project_groups = await ReportController.getProjectGroupsOverview(selectedProjectGroups, request.dateFrom, request.dateTo, sessionToken);
                                    response.success = true;
                                    break;

                                // USERS
                                case Requests.MessageTypes.AdminCreateUser:
                                    if (sessionToken.joint_can_manage_users) {
                                        if (request.user.password == '') {
                                            response.success = false;
                                            response.error = i18n.localize('empty_password');
                                        }
                                        else if (!await UsersController.checkForUniqueName(request.user.username)) {
                                            response.success = false;
                                            response.error = i18n.localize('name_exist');
                                        }
                                        else {
                                            request.user.password = CryptoUtils.decipherToken(request.user.password, request.timestamp);

                                            if (PasswordHandler.testPassStrength(request.user.password)) {
                                                response.success = false;
                                                response.error = i18n.localize('unacceptable_password');
                                            }
                                            else {
                                                response.users = await UsersController.create(request.user, sessionToken);
                                                response.success = true;
                                                auditTrail.message_content = i18n.localize('user_created', {user: request.user});
                                                AuditTrailController.create(auditTrail, sessionToken);
                                            }
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewUsers:
                                    if (sessionToken.joint_can_manage_users) {
                                        response.users = await UsersController.getAll(request.query, request.offset, request.limit, request.sort);
                                        response.success = true;
                                        if (request.export === '1') {
                                            response.file = '';
                                            response.file = FileUtils.createExportViewFile(UsersController.getArrayForExport(response.users));
                                            response.filename = i18n.localize('users_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                            if (response.file === '') {
                                                response.success = false;
                                                response.error = i18n.localize('server_error');
                                            }
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewUser:
                                    if (sessionToken.joint_can_manage_users || sessionToken.user.id == request.user_id) {
                                        response.user = await UsersController.getById(request.user_id);
                                        response.success = true;
                                        if (response.user === false) {
                                            response.success = false;
                                            response.error = i18n.localize('access_denied');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminEditUser:
                                    if (sessionToken.joint_can_manage_users || sessionToken.user.id === request.user.id) {
                                        user = await UsersController.getById(request.user.id);
                                        if (user.username !== request.user.username && !(await UsersController.checkForUniqueName(request.user.username))) {
                                            response.success = false;
                                            response.error = i18n.localize('duplicated_username');
                                            sessionToken = await SessionTokensController.getByHash(sessionToken.hash);
                                        }
                                        else {
                                            let allow_edit = true;
                                            if (request.user.password !== '') {
                                                request.user.password = CryptoUtils.decipherToken(request.user.password, request.timestamp);
                                                if (PasswordHandler.testPassStrength(request.user.password)) {
                                                    response.success = false;
                                                    response.error = i18n.localize('unacceptable_password');
                                                    allow_edit = false;
                                                }
                                            }
                                            if (allow_edit) {

                                                if (sessionToken.user.id === request.user.id) {
                                                    let isRolesChanged = (sessionToken.user.roles.length !== request.user.roles.length);
                                                    if (!isRolesChanged) {
                                                        for (let role of sessionToken.user.roles) {
                                                            let isRoleFound = false;
                                                            for (let roleOther of request.user.roles) {
                                                                if (role.id === roleOther.id) {
                                                                    isRoleFound = true;
                                                                    break;
                                                                }
                                                            }
                                                            if (!isRoleFound) {
                                                                isRolesChanged = true;
                                                            }
                                                        }
                                                    }
                                                    if (
                                                        isRolesChanged ||
                                                        sessionToken.user.can_access_admin_console !== request.user.can_access_admin_console ||
                                                        sessionToken.user.can_manage_users !== request.user.can_manage_users ||
                                                        sessionToken.user.can_access_all_project_groups !== request.user.can_access_all_project_groups ||
                                                        sessionToken.user.can_access_all_instruments !== request.user.can_access_all_instruments
                                                    ) {
                                                        response.success = false;
                                                        response.error = i18n.localize('cant_change_access_level_yourself');
                                                        allow_edit = false;
                                                    }
                                                }

                                                if (allow_edit) {
                                                    response.user = await UsersController.update(request.user, request.sessionToken.user);
                                                    response.success = true;
                                                    auditTrail.message_content = i18n.localize('user_updated', {user: request.user});
                                                    AuditTrailController.create(auditTrail, sessionToken);
                                                }
                                            }

                                            if (!allow_edit) {
                                                auditTrail.event_type_id = EventTypes.Error;
                                                auditTrail.message_content = i18n.localize('user_failed_to_edit', {
                                                    user: request.user,
                                                    error: response.error
                                                });
                                                AuditTrailController.create(auditTrail, sessionToken);
                                            }
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminDeleteUser:
                                    if (sessionToken.joint_can_manage_users) {
                                        let del_user = await UsersController.getById(request.user_id, {is_details: false});
                                        if (request.user_id === request.sessionToken.user.id) {
                                            response.success = false;
                                            response.error = i18n.localize('cant_delete_yourself');
                                        }
                                        else {
                                            response.user = await UsersController.delete(request.user_id, request.sessionToken.user);
                                            response.success = true;
                                            auditTrail.message_content = i18n.localize('user_deleted', {user: del_user});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;

                                case Requests.MessageTypes.PasswordExpired:
                                    user = await UsersController.getById(sessionToken.user.id);
                                    let new_pass = CryptoUtils.decipherToken(request.new_password, request.timestamp);
                                    let old_pass = CryptoUtils.decipherToken(request.old_password, request.timestamp);
                                    let new_pass_hash = UsersController.getHash(user.username, new_pass);
                                    let old_pass_hash = UsersController.getHash(user.username, old_pass);

                                    if (old_pass_hash != user.password) {
                                        response.success = false;
                                        response.error = i18n.localize('wrong_old_pass');
                                    }
                                    else if (new_pass_hash != user.password) {
                                        await UsersController.updatePassword(user, new_pass);
                                        response.success = true;
                                        sessionToken.password_expired = false;
                                        auditTrail.message_content = i18n.localize('update_expired_password', {user: user});
                                        AuditTrailController.create(auditTrail, sessionToken);
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('old_and_new_pass_equals');
                                    }
                                    break;


                                // AUDITTRAIL
                                case Requests.MessageTypes.AdminViewAuditTrail:
                                    response.results = await AuditTrailController.getAll(request.query, request.offset, request.limit, request.sort, sessionToken);
                                    response.success = true;
                                    if (request.export === '1') {
                                        response.file = '';
                                        response.file = FileUtils.createExportViewFile(AuditTrailController.getArrayForExport(response.results, devTypeName, eventTypeName));
                                        response.filename = i18n.localize('audit_trail_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                        if (response.file === '') {
                                            response.success = false;
                                            response.error = i18n.localize('server_error');
                                        }
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewAuditTrailDetail:
                                    response.item = await AuditTrailController.getById(request.audit_trail_id, sessionToken);
                                    response.success = true;
                                    break;

                                // ROLES
                                case Requests.MessageTypes.AdminCreateRole:
                                    if (sessionToken.joint_can_manage_users) {
                                        if (await RolesController.checkForUniqueRole(request.role.name)) {
                                            response.roles = await RolesController.create(request.role, request.sessionToken);
                                            response.success = true;
                                            auditTrail.message_content = i18n.localize('role_created', {role: request.role});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                        else {
                                            response.success = false;
                                            response.error = i18n.localize('duplicated_role_name');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;

                                case Requests.MessageTypes.AdminViewRoles:
                                    if (sessionToken.joint_can_manage_users) {
                                        response.roles = await RolesController.getAll(request.query, request.offset, request.limit, request.sort);
                                        response.success = true;
                                        if (request.export === '1') {
                                            response.file = '';
                                            response.file = FileUtils.createExportViewFile(RolesController.getArrayForExport(response.roles));
                                            response.filename = i18n.localize('roles_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                            if (response.file === '') {
                                                response.success = false;
                                                response.error = i18n.localize('server_error');
                                            }
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;

                                case Requests.MessageTypes.AdminViewRole:
                                    if (sessionToken.joint_can_manage_users) {
                                        response.role = await RolesController.getById(request.role_id);
                                        response.success = true;
                                        if (response.role === false) {
                                            response.success = false;
                                            response.error = i18n.localize('access_denied');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminEditRole:
                                    if (sessionToken.joint_can_manage_users) {
                                        let role = await RolesController.getById(request.role.id);
                                        let allow_changes = false;
                                        if (role.name !== request.role.name && !(await RolesController.checkForUniqueRole(request.role.name))) {
                                            response.success = false;
                                            response.error = i18n.localize('duplicated_role_name');
                                        }
                                        else {
                                            let isThisYourRole = false;
                                            for (let role of sessionToken.user.roles) {
                                                if (role.id === request.role.id) {
                                                    isThisYourRole = true;
                                                    break;
                                                }
                                            }

                                            if (!isThisYourRole) {
                                                response.roles = await RolesController.update(request.role, sessionToken);
                                                response.success = true;
                                                auditTrail.message_content = i18n.localize('role_updated', {role: role});
                                            }
                                            else {
                                                response.success = false;
                                                auditTrail.event_type_id = EventTypes.Error;
                                                auditTrail.message_content = response.error = i18n.localize('cant_change_access_level_yourself');
                                            }
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminDeleteRole:
                                    if (sessionToken.joint_can_manage_users) {
                                        if (await UsersRolesController.isRoleUsing(request.role_id)) {
                                            let del_role = await RolesController.getById(request.role_id, {is_details: false});
                                            response.role = await RolesController.delete(request.role_id, request.sessionToken.user);
                                            response.success = true;
                                            auditTrail.message_content = i18n.localize('role_deleted', {role: del_role});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                        else {
                                            response.success = false;
                                            response.error = i18n.localize('role_using');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;


                                // INSTRUMENTS
                                case Requests.MessageTypes.AccessInstrument:
                                    let vendor = false;

                                    if (process.env.ENV !== 'development') {
                                        if (!sessionToken.user.can_access_all_instruments) {
                                            vendor = request.vendor;
                                        }
                                    }

                                    var instruments_ids = await InstrumentsController.getByPcNames(request.pc_names, vendor);
                                    response.success = false;

                                    if (instruments_ids && instruments_ids.length > 0) {

                                        if (process.env.ENV !== 'development') {
                                            if (sessionToken.user.can_access_all_instruments) {
                                                response.success = true;
                                            }
                                        }

                                        for (var each of sessionToken.user.instruments) {
                                            if (instruments_ids.indexOf(each.id) > -1) {
                                                response.success = true;
                                                break;
                                            }
                                        }

                                        for (let u_r of sessionToken.user.roles) {

                                            if (process.env.ENV !== 'development') {
                                                if (u_r.can_access_all_instruments) {
                                                    response.success = true;
                                                    break;
                                                }
                                            }

                                            for (let ins of u_r.instruments) {
                                                if (instruments_ids.indexOf(ins.id) > -1) {
                                                    response.success = true;
                                                    break;
                                                }
                                            }
                                        }

                                        if (response.success) {
                                            response.instrument = await InstrumentsController.getById(instruments_ids[0], sessionToken);
                                        }
                                    }
                                    if (!response.success) {
                                        if (!instruments_ids.length) {
                                            //Add new instrument template
                                            response.error = i18n.localize('access_error_missing_instrument', {pc_name: request.pc_names[0]});
                                        }
                                        else {
                                            response.error = i18n.localize('access_error_instrument', {pc_name: request.pc_names[0]});
                                        }
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewInstruments:
                                    response.instruments = await InstrumentsController.getAll(request.query, request.offset, request.limit, request.sort, sessionToken);
                                    response.success = true;
                                    if (request.export === '1') {
                                        response.file = '';
                                        response.file = FileUtils.createExportViewFile(InstrumentsController.getArrayForExport(response.instruments, vendorTypeName));
                                        response.filename = i18n.localize('instruments_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                        if (response.file === '') {
                                            response.success = false;
                                            response.error = i18n.localize('server_error');
                                        }
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewInstrument:
                                    response.instrument = await InstrumentsController.getById(request.instrument_id, sessionToken);
                                    response.success = true;
                                    if (response.instrument === false) {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminCreateInstrument:
                                    if (sessionToken.joint_can_access_all_instruments) {
                                        if (await InstrumentsController.checkForDuplicatedProps(request.instrument)) {
                                            response.success = false;
                                            response.error = i18n.localize('such_instrument_already_exist');
                                        }
                                        else if (await InstrumentsController.checkForUniqueName(request.instrument.name)) {
                                            response.instrument = await InstrumentsController.create(request.instrument, sessionToken);
                                            response.success = true;
                                            auditTrail.instrument_id = response.instrument.id
                                            auditTrail.message_content = i18n.localize('instrument_created', {instrument: request.instrument});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                        else {
                                            response.success = false;
                                            response.error = i18n.localize('duplicated_instrument_name');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminEditInstrument:
                                    if (sessionToken.joint_can_access_all_instruments) {
                                        let inst_edit = await InstrumentsController.getById(request.instrument.id, sessionToken);
                                        if (await InstrumentsController.checkForDuplicatedProps(request.instrument)) {
                                            response.success = false;
                                            response.error = i18n.localize('such_instrument_already_exist');
                                        }
                                        else if (inst_edit.name != request.instrument.name && !(await InstrumentsController.checkForUniqueName(request.instrument.name))) {
                                            response.success = false;
                                            response.error = i18n.localize('duplicated_instrument_name');
                                        }
                                        else {
                                            response.instrument = await InstrumentsController.update(request.instrument, sessionToken);
                                            response.success = true;
                                            auditTrail.instrument_id = response.instrument.id
                                            auditTrail.message_content = i18n.localize('instrument_updated', {instrument: request.instrument});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminDeleteInstrument:
                                    if (sessionToken.joint_can_access_all_instruments) {
                                        if (!await ProjectController.isInstrumentUsing(request.instrument_id)) {
                                            response.success = false;
                                            response.error = i18n.localize('instrument_using_in_project');
                                        }
                                        else if (!await UsersController.isInstrumentUsing(request.instrument_id)) {
                                            response.success = false;
                                            response.error = i18n.localize('instrument_using_in_users');
                                        }
                                        else if (!await RolesController.isInstrumentUsing(request.instrument_id)) {
                                            response.success = false;
                                            response.error = i18n.localize('instrument_using_in_roles');
                                        }
                                        else {
                                            let del_inst = await InstrumentsController.getById(request.instrument_id, sessionToken);
                                            await InstrumentsController.delete(request.instrument_id, sessionToken);
                                            response.success = true;
                                            auditTrail.instrument_id = response.instrument.id
                                            auditTrail.message_content = i18n.localize('instrument_deleted', {instrument: del_inst});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;

                                // PROJECT GROUPS
                                case Requests.MessageTypes.AdminViewProjectGroups:
                                    response.projectGroups = await ProjectGroupsController.getAll(request.query, request.offset, request.limit, request.sort, sessionToken);
                                    response.success = true;
                                    if (request.export === '1') {
                                        response.file = '';
                                        response.file = FileUtils.createExportViewFile(ProjectGroupsController.getArrayForExport(response.projectGroups, devTypeName));
                                        response.filename = i18n.localize('project_groups_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                        if (response.file === '') {
                                            response.success = false;
                                            response.error = i18n.localize('server_error');
                                        }
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewProjectGroup:
                                    response.projectGroup = await ProjectGroupsController.getById(request.project_group_id, sessionToken);
                                    response.success = true;
                                    if (response.projectGroup === false) {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AvailableProjectGroups:
                                    response.projectGroups = await ProjectGroupsController.getAvailableProjects(sessionToken);
                                    response.success = true;
                                    break;
                                case Requests.MessageTypes.AdminCreateProjectGroups:
                                    if (sessionToken.joint_can_access_all_project_groups) {
                                        if (await ProjectGroupsController.checkForUniqueName(request.projectGroup.name)) {
                                            response.projectGroup = ProjectGroupsController.create(request.projectGroup, sessionToken);
                                            response.success = true;
                                            auditTrail.project_group_id = response.projectGroup.id;
                                            auditTrail.message_content = i18n.localize('project_group_created', {projectGroup: request.projectGroup});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                        else {
                                            response.success = false;
                                            response.error = i18n.localize('project_group_name_already_exists');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminEditProjectGroups:
                                    if (sessionToken.joint_can_access_all_project_groups) {
                                        let e_pg = await ProjectGroupsController.getById(request.projectGroup.id, sessionToken);
                                        if (e_pg.name == request.projectGroup.name || await ProjectGroupsController.checkForUniqueName(request.projectGroup.name)) {
                                            response.projectGroup = ProjectGroupsController.update(request.projectGroup, sessionToken);
                                            response.success = true;
                                            auditTrail.project_group_id = response.projectGroup.id;
                                            auditTrail.message_content = i18n.localize('project_group_updated', {projectGroup: request.projectGroup});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                        else {
                                            response.success = false;
                                            response.error = i18n.localize('project_group_name_already_exists');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminDeleteProjectGroups:
                                    if (sessionToken.joint_can_access_all_project_groups) {
                                        if (await ProjectController.isProjectGroupUsing(request.project_group_id)) {
                                            let del_pg = await ProjectGroupsController.getById(request.project_group_id, sessionToken);
                                            await ProjectGroupsController.delete(request.project_group_id, sessionToken.user);
                                            response.success = true;
                                            auditTrail.message_content = i18n.localize('project_group_deleted', {projectGroup: del_pg});
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }
                                        else {
                                            response.success = false;
                                            response.error = i18n.localize('project_group_using');
                                        }
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;

                                // PROJECTS
                                case Requests.MessageTypes.AdminViewProjects:
                                    response.projects = await ProjectController.getAll(request.query, request.offset, request.limit, request.sort, sessionToken);
                                    response.success = true;
                                    if (request.export === '1') {
                                        response.file = '';
                                        response.file = FileUtils.createExportViewFile(ProjectController.getArrayForExport(response.projects, devTypeName));
                                        response.filename = i18n.localize('projects_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                        if (response.file === '') {
                                            response.success = false;
                                            response.error = i18n.localize('server_error');
                                        }
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewProject:
                                    response.project = await ProjectController.getById(request.project_id, sessionToken);
                                    response.success = true;
                                    if (response.project === false) {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AccessProject:
                                    project = await ProjectController.getById(request.project_id, sessionToken);
                                    response.success = false;
                                    if (project && project.id) {
                                        if (sessionToken.user.can_access_all_project_groups) {
                                            response.success = true;
                                        }
                                        for (let projectGroup of sessionToken.user.projectGroups) {
                                            if (projectGroup.id === project.project_group_id) {
                                                response.success = true;
                                                break;
                                            }
                                        }

                                        if (!response.success) {
                                            for (let u_r of sessionToken.user.roles) {
                                                if (u_r.can_access_all_project_groups) {
                                                    response.success = true;
                                                    break;
                                                }

                                                for (let projectGroup of u_r.projectGroups) {
                                                    if (projectGroup.id === project.project_group_id) {
                                                        response.success = true;
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                    }

                                    //respond with project so that its name can be updated
                                    if(response.success) {
                                        response.project = project;
                                    }

                                    auditTrail.event_type_id = EventTypes.Info;
                                    auditTrail.project_id = project.id;
                                    auditTrail.project_group_id = project.project_group_id;

                                    if (!response.success) {
                                        response.error = i18n.localize('access_error_project');
                                        auditTrail.message_content = i18n.localize('project_accessed_failed', {project: project});
                                    }
                                    else {
                                        auditTrail.message_content = i18n.localize('project_accessed', {project: project});
                                    }
                                    AuditTrailController.create(auditTrail, sessionToken);
                                    break;
                                case Requests.MessageTypes.AdminEditProject:
                                    if (await ProjectController.isUniqueNameInProjectGroup(request.project)) {
                                        await ProjectController.update(request.project, sessionToken);
                                        response.success = true;
                                        auditTrail.project_id = request.project.id;
                                        auditTrail.project_group_id = request.project.project_group_id;
                                        auditTrail.message_content = i18n.localize('project_updated', {project: request.project});
                                        AuditTrailController.create(auditTrail, sessionToken);
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('duplicate_project_name');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminDeleteProject:
                                    if (sessionToken.joint_can_access_all_project_groups) {
                                        let del_pr = await ProjectController.getById(request.project_id, sessionToken);
                                        await ProjectController.delete(request.project_id, sessionToken.user);
                                        response.success = true;
                                        auditTrail.project_id = del_pr.id;
                                        auditTrail.project_group_id = del_pr.project_group_id;
                                        auditTrail.message_content = i18n.localize('project_deleted', {project: del_pr});
                                        AuditTrailController.create(auditTrail, sessionToken);
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.CheckProjectName:
                                    response.success = (await ProjectController.isUniqueNameInProjectGroup(request.project));
                                    if (!response.success)
                                        response.error = i18n.localize('duplicate_project_name');
                                    break;
                                case Requests.MessageTypes.AdminDownloadProjectFile:
                                case Requests.MessageTypes.AdminDownloadReportFile:
                                    if (request.messageType == Requests.MessageTypes.AdminDownloadProjectFile) {
                                        file_type = 'project';
                                        serverFolder = 'Projects';
                                        item2download = await ProjectController.getById(request.item_id, sessionToken);
                                    }
                                    else {
                                        file_type = 'report';
                                        serverFolder = 'Reports';
                                        file_ext = FileUtils.getExtensionByDevelopmentType(DevelopmentTypes.ReportViewer);
                                        item2download = await ReportController.getById(request.item_id, sessionToken);
                                    }

                                    let lastAuditTrialWithFile = {};
                                    if (request.downloadById === undefined) {
                                        lastAuditTrialWithFile = await AuditTrailController.getLastProjectRevFile(request.item_id, file_type);
                                    }
                                    else {
                                        lastAuditTrialWithFile = await AuditTrailController.getById(request.audit_trail_id, sessionToken);
                                    }

                                    if (lastAuditTrialWithFile === false) {
                                        response.success = false;
                                        response.error = i18n.localize('no_file_connected_with_current_item');
                                    }
                                    else {
                                        if (!file_ext) {
                                            file_ext = FileUtils.getExtensionByDevelopmentType(lastAuditTrialWithFile.dev_type_id);
                                        }
                                        response.filename = item2download.name + '_' + lastAuditTrialWithFile.id + '.' + file_ext;
                                        response.file = FileUtils.downloadFile(lastAuditTrialWithFile.id, file_ext, serverFolder);
                                        if (response.file === false) {
                                            response.success = false;
                                            response.error = i18n.localize('cant_find_file_on_a_server');
                                        }
                                        else {
                                            response.success = true;
                                        }

                                    }
                                    break;
                                case Requests.MessageTypes.CreateProject:
                                case Requests.MessageTypes.UpdateProject:

                                    if (request.messageType === Requests.MessageTypes.UpdateProject ||
                                        await ProjectController.isUniqueNameInProjectGroup(request.project)) {

                                        let auditMessage = 'event_create_project';
                                        if (request.messageType === Requests.MessageTypes.CreateProject ||
                                            !request.project.id) {
                                            response.project = await ProjectController.create(request.project, sessionToken);
                                        } else {
                                            response.project = await ProjectController.update(request.project, sessionToken);
                                            auditMessage = 'event_update_project';
                                        }

                                        auditTrail.project_id = request.project.id;
                                        auditTrail.project_group_id = request.project.project_group_id;
                                        auditTrail.project_file_revision = true;
                                        if(request.project) {
                                            auditTrail.dev_type_id = request.project.dev_type_id;
                                        }
                                        auditTrail.message_content = i18n.localize(auditMessage, {project: request.project});
                                        auditTrail = await AuditTrailController.create(auditTrail, sessionToken);

                                        // Store file using audit trail ID
                                        FileUtils.saveFile(auditTrail.id, FileUtils.getExtensionByDevelopmentType(request.project.dev_type_id), 'Projects', request.file);

                                        response.success = true;
                                    } else {
                                        response.success = false;
                                        response.error = i18n.localize('duplicate_project_name');
                                    }
                                    break;


                                // REPORTS
                                case Requests.MessageTypes.AdminViewReports:
                                    response.reports = await ReportController.getAll(request.query, request.offset, request.limit, request.sort, sessionToken);
                                    response.success = true;
                                    if (request.export === '1') {
                                        response.file = '';
                                        response.file = FileUtils.createExportViewFile(ReportController.getArrayForExport(response.reports, devTypeName));
                                        response.filename = i18n.localize('reports_list') + '_' + (new Date()).toISOString().slice(0, 10) + '.xlsx';
                                        if (response.file === '') {
                                            response.success = false;
                                            response.error = i18n.localize('server_error');
                                        }
                                    }
                                    break;
                                case Requests.MessageTypes.AdminViewReport:
                                    response.item = await ReportController.getById(request.item_id, sessionToken);
                                    response.success = true;
                                    if (response.item === false) {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminEditReport:
                                    if (await ReportController.isUniqueNameInReports(request.item)) {
                                        await ReportController.update(request.item, sessionToken);
                                        response.success = true;
                                        auditTrail.project_id = request.item.project_id;
                                        auditTrail.instrument_id = request.item.instrument_id;
                                        auditTrail.report_id = request.item.id;
                                        auditTrail.message_content = i18n.localize('report_updated', {report: request.item});
                                        AuditTrailController.create(auditTrail, sessionToken);
                                    }
                                    else {
                                        response.success = false;
                                        response.error = i18n.localize('duplicated_report_name');
                                    }
                                    break;
                                case Requests.MessageTypes.AdminDeleteReport:
                                    let del_rep = await ReportController.getById(request.item_id, sessionToken);
                                    await ReportController.delete(request.item_id, sessionToken.user);
                                    response.success = true;
                                    auditTrail.project_id = del_rep.project_id;
                                    auditTrail.instrument_id = del_rep.instrument_id;
                                    auditTrail.report_id = del_rep.id;
                                    auditTrail.message_content = i18n.localize('report_deleted', {report: del_rep});
                                    AuditTrailController.create(auditTrail, sessionToken);
                                    break;
                                case Requests.MessageTypes.AccessReport:
                                    response.success = false;
                                    if (request.report) {
                                        report = null;
                                        if (request.report.hash) {
                                            if(await ReportController.isExistingHash(request.report.hash)) {
                                                report = await ReportController.getByHash(request.report.hash, sessionToken);
                                                if (report !== undefined) {
                                                    report = await ReportController.getById(report.id, sessionToken); //add detailed info
                                                } else {
                                                    report = null;
                                                }
                                            }
                                            else
                                            {
                                                response.error_code = 'missing_report_hash';
                                            }
                                        }
                                        else if (request.report.id) {
                                            report = await ReportController.getById(
                                                request.report.id,
                                                sessionToken
                                            );
                                        }

                                        if (report) {
                                            response.report = report;

                                            if (report.project) {
                                                if (sessionToken.user.can_access_all_project_groups) {
                                                    response.success = true;
                                                }
                                                for (let projectGroup of sessionToken.user.projectGroups) {
                                                    if (projectGroup.id === report.project.project_group_id) {
                                                        response.success = true;
                                                        break;
                                                    }
                                                }

                                                if (!response.success) {
                                                    for (let u_r of sessionToken.user.roles) {
                                                        if (u_r.can_access_all_project_groups) {
                                                            response.success = true;
                                                            break;
                                                        }

                                                        for (let projectGroup of u_r.projectGroups) {
                                                            if (projectGroup.id === report.project.project_group_id) {
                                                                response.success = true;
                                                                break;
                                                            }
                                                        }
                                                    }
                                                }
                                            }

                                            auditTrail.report_id = report.id;
                                            auditTrail.event_type_id = EventTypes.Info;
                                            auditTrail.project_id = report.project.id;
                                            auditTrail.project_group_id = report.project.project_group_id;

                                            if (!response.success) {
                                                response.error = i18n.localize('access_error_project');
                                                auditTrail.message_content = i18n.localize('report_accessed', {report: report});
                                            }
                                            else {
                                                auditTrail.message_content = i18n.localize('report_accessed', {report: report});
                                            }
                                            AuditTrailController.create(auditTrail, sessionToken);
                                        }

                                    }
                                    break;


                                case Requests.MessageTypes.AdminViewProjectReportDetails:
                                    response.items = await AuditTrailController.getByAssoictedId(request.item_type, request.item_id, request.offset, request.limit, sessionToken);
                                    response.success = true;
                                    if (response.items === false) {
                                        response.success = false;
                                        response.error = i18n.localize('access_denied');
                                    }
                                    break;


                                // Offline tokens
                                case Requests.MessageTypes.StartProjectExecution:
                                    sessionToken.offline_project_id = request.offline_project_id;
                                    sessionToken.offline_instrument_id = request.offline_instrument_id;
                                    await SessionTokensController.update(sessionToken);

                                    project = await ProjectController.getById(request.offline_project_id, sessionToken);
                                    auditTrail.message_content = i18n.localize('event_start_project', {project: project});
                                    if(project) {
                                        auditTrail.dev_type_id = project.dev_type_id;
                                        auditTrail.project_id = project.id;
                                    }
                                    auditTrail = await AuditTrailController.create(auditTrail, sessionToken);

                                    response.success = true;
                                    break;

                                // LOGOUT
                                case Requests.MessageTypes.Logout:
                                    response.success = true;
                                    break;
                            }
                        }
                        else {
                            if (request.messageType === Requests.MessageTypes.AdminViewOverview) {
                                //Before login do not show anything
                                response.data_dev_type = [];
                                response.data_users_values = [];
                                response.data_instruments = [];
                                response.data_runs = [];
                                response.data_project_groups = [];
                                response.success = true;
                            }
                        }

                        if (!sessionToken.is_valid && ('offline_hash' in request)) {
                            //try to load offline token
                            sessionToken = await SessionTokensController.getByHash(request.offline_hash);

                            if (!sessionToken && process.env.ENV === 'development') {
                                console.log('sessionToken not found by hash', request.offline_hash);
                            }

                            if (sessionToken) {
                                if (request.report && request.report.hash) {
                                    report = await ReportController.getByHash(request.report.hash, sessionToken);

                                    if (request.messageType === Requests.MessageTypes.CreateReport || // Request to crate new report
                                        (report &&
                                        sessionToken.offline_project_id > 0 && sessionToken.offline_instrument_id > 0 && //update existing
                                        sessionToken.offline_project_id === report.project_id &&
                                        sessionToken.offline_instrument_id === report.instrument_id)) {

                                        // Data can be stored with session token marked for offline use / used for report service
                                        sessionToken.is_valid = true;
                                    }
                                    else {
                                        if (process.env.ENV === 'development') {
                                            console.log('offline token invalid', sessionToken, report);
                                        }
                                    }
                                }
                            }
                            else {
                                sessionToken = SessionTokensController.Default;
                            }
                        }

                        // Data can be stored with session token marked for offline use / used for report service
                        if (sessionToken.is_valid && response.messageType !== Requests.MessageTypes.InvalidRepeatedRequest) {

                            // For this section time must be taken from request, because they come in with delay
                            // Temporary add offline_timestamp, then must be deleted before sending response
                            sessionToken.offline_timestamp = {
                                date: (new Date(request.timestamp))
                            };
                            sessionToken.offline_timestamp.time = DateTimeUtils.getTimeLocalString(sessionToken.offline_timestamp.date);


                            // Restore appropriate parts of report structure in offline mode
                            if (('report' in request)) {

                                if(!request.report.project_id) {
                                    if(sessionToken.offline_project_id) {
                                        request.report.project_id = sessionToken.offline_project_id;
                                        let project = await ProjectController.getById(request.report.project_id, sessionToken);
                                        if(project) {
                                            request.report.project_group_id = project.project_group_id;
                                        }
                                    }
                                }
                                if(!request.report.instrument_id) {
                                    if(sessionToken.offline_instrument_id) {
                                        request.report.instrument_id = sessionToken.offline_instrument_id;
                                    }
                                }

                                if (('hash' in request.report) && request.report.hash.length > 0 && !request.report.id) {
                                    if(request.messageType !== Requests.MessageTypes.CreateReport) {
                                        if(await ReportController.isExistingHash(request.report.hash)) {
                                            report = await ReportController.getByHash(request.report.hash, sessionToken);
                                            if (report) {
                                                request.report.id = report.id;
                                                CodeUtils.overrideProperties(request.report, report);
                                                if (report.project) {
                                                    request.report.project = report.project;
                                                }
                                            }
                                        }
                                        else {
                                            response.error_code = 'missing_report_hash';
                                        }
                                    }
                                }
                                else if (!('project' in request.report) && request.report.id) {
                                    report = await ReportController.getById(request.report.id, sessionToken);
                                    if (report) {
                                        request.report.id = report.id;
                                        CodeUtils.overrideProperties(request.report, report);
                                        if(report.project) {
                                            request.report.project = report.project;
                                        }
                                    }
                                }

                                if (!('project' in request.report) && request.report.project_id) {
                                    project = await ProjectController.getById(request.report.project_id, sessionToken);
                                    if (project) {
                                        request.project = {};
                                        request.project.id = project.id;
                                        CodeUtils.overrideProperties(request.project, project);
                                        request.report.project = project;
                                    }
                                }

                                // Prepare audit structure
                                auditTrail = AuditTrailController.Default;
                                if (request.report.project) {
                                    auditTrail.event_type_id = EventTypes.Info;
                                    auditTrail.report_id = request.report.id;
                                    auditTrail.project_id = request.report.project.id;
                                    auditTrail.instrument_id = request.report.instrument_id;
                                    auditTrail.project_group_id = request.report.project.project_group_id;
                                    auditTrail.dev_type_id = request.report.project.dev_type_id;
                                }

                                auditTrail.report_file_revision = false;
                                try {
                                    if (request.file !== undefined) {
                                        if (request.file !== false) {
                                            auditTrail.report_file_revision = true;
                                        }
                                    }
                                }
                                catch(exc) {}
                            }

                            switch (request.messageType) {
                                case Requests.MessageTypes.AuditTrail:

                                    if (sessionToken.is_valid) {
                                        //Sync call
                                        //Custom audit trail, not related to reports when token is valid
                                        auditTrail = await AuditTrailController.create(request.auditTrail, sessionToken);

                                        response.success = true;
                                    } else {
                                        //audit trail from report
                                        if (request.auditTrail.event_type_id) {
                                            auditTrail.event_type_id = request.auditTrail.event_type_id;
                                        }
                                        auditTrail.message_content = request.auditTrail.message_content;
                                        auditTrail = await AuditTrailController.create(auditTrail, sessionToken);

                                        response.success = true;
                                    }
                                    break;

                                case Requests.MessageTypes.FinishProjectExecution:

                                    project = await ProjectController.getById(request.offline_project_id, sessionToken);

                                    auditTrail.message_content = i18n.localize('event_finish_project', {project: project});
                                    auditTrail.event_type_id = EventTypes.Info;

                                    if(project !== undefined) {
                                        auditTrail.project_id = project.id;
                                        auditTrail.dev_type_id = project.dev_type_id;
                                    }

                                    auditTrail = await AuditTrailController.create(auditTrail, sessionToken);

                                    sessionToken.offline_project_id = 0;
                                    sessionToken.offline_instrument_id = 0;
                                    await SessionTokensController.update(sessionToken);

                                    response.success = true;
                                    break;

                                case Requests.MessageTypes.CreateReport:
                                case Requests.MessageTypes.UpdateReport:

                                    let auditMessage = 'event_create_report';
                                    if (request.messageType === Requests.MessageTypes.CreateReport || !request.report.id) {
                                        response.report = await ReportController.create(request.report, sessionToken);
                                        logger.info('created:' +JSON.stringify(response.report));
                                    } else {
                                        for(let key in request.report) {
                                            report[key] = request.report[key];
                                        }
                                        response.report = await ReportController.update(report, sessionToken);
                                        auditMessage = 'event_update_report';
                                    }

                                    auditTrail.event_type_id = EventTypes.Info;
                                    if(response.report.project) {
                                        auditTrail.dev_type_id = response.report.project.dev_type_id;
                                    }

                                    if (request.event_type_id) {
                                        // When saving in report viewer
                                        auditTrail.event_type_id = request.event_type_id;
                                    }
                                    if (request.dev_type_id) {
                                        // When saving in report viewer
                                        auditTrail.dev_type_id = request.dev_type_id;
                                    }

                                    auditTrail.instrument_id = response.report.instrument_id;
                                    auditTrail.report_id = response.report.id;
                                    auditTrail.project_id = response.report.project.id;
                                    auditTrail.project_group_id = response.report.project.project_group_id;
                                    auditTrail.message_content = i18n.localize(auditMessage, {report: response.report});

                                    auditTrail = await AuditTrailController.create(auditTrail, sessionToken);

                                    if (auditTrail.report_file_revision && request.file) {
                                        // Store file using audit trail ID
                                        FileUtils.saveFile(auditTrail.id, 'rptx', 'Reports', request.file);
                                    }

                                    // Update audit trail to link to report existing tails
                                    await ReportController.updateAuditTrailByOfflineHash(request.report);

                                    response.success = true;

                                    break;
                            }
                        }

                        // logout do not require is_valid
                        if (request.messageType === Requests.MessageTypes.Login) {
                            logger.info('login');
                            request.password = CryptoUtils.decipherToken(request.password, request.timestamp);
                            let user = await UsersController.getByUsernamePassword(request.username, request.password);
                            if (user.id > 0 && user.is_active && !user.is_deleted) {
                                sessionToken = await SessionTokensController.create(user, request);
                                response.success = true;

                                logger.info('login successful');
                                sessionToken.password_expired = PasswordHandler.passExpireCheck(user.password_date, user.password_time, settings);

                                // Register successful login attempt
                                auditTrail.message_content = i18n.localize('event_login_successful', {request: request});
                                if ('dev_type_id' in request) {
                                    auditTrail.dev_type_id = request.dev_type_id;
                                }
                                await AuditTrailController.create(auditTrail, sessionToken);
                            } else {
                                logger.warn('wrong login');
                                // Register unsuccessful login attempt
                                auditTrail.event_type_id = AuditTrailController.EventTypes.Error;
                                auditTrail.message_content = i18n.localize('event_login_unsuccessful', {request: request});
                                if ('dev_type_id' in request) {
                                    auditTrail.dev_type_id = request.dev_type_id;
                                }
                                await AuditTrailController.create(auditTrail, sessionToken);
                            }
                        }

                    }
                    catch (exc) {
                        logger.error(exc);
                        LoggerUtils.logRequest(request);
                    }


                    response.messageType = request.messageType;
                    if (!sessionToken.is_valid) {
                        response.messageType = Requests.MessageTypes.InvalidSession;
                        response.success = false;
                    }

                    // remove temporary structures
                    if ('offline_timestamp' in sessionToken) {
                        delete sessionToken.offline_timestamp;
                    }

                    if(sessionToken.is_valid !== undefined) {
                        response.sessionToken = sessionToken;
                    }
                    response.timestamp = (new Date()).getTime();

                    // cannot use mdt on full request, will cause mem crash on x86
                    let file = request.file;
                    if (file) {
                        delete request.file;
                    }
                    response.checksum = CryptoUtils.md5('chk_back' + JSON.stringify(response) + response.timestamp.toString());
                    if (file) {
                        request.file = file;
                    }
                }
            }

            if (process.env.ENV === 'development') {
                logger.debug(`Response: ${EnumUtils.Stringify(response.messageType, RequestMessageTypes)} success: ${response.success}`);
                //LoggerUtils.logRequest(response);
            }
        }
        catch (exc) {
            logger.error(exc);
        }

        httpResponse.send(response);
    }
}
