const logger = require('../common/Logger');
const pgAsync = require('../utils/Database');
const SQL = require('pg-async').SQL;
const CodeUtils = require('../utils/CodeUtils');

module.exports = class InstrumentIpAddressesController{
    static get Default() {
        return {
            id: 0,
            host: '',
            instrument_id: 0,

            is_deleted: false,
            modified_user_id: false,
            modified_time: false,
            modified_date: false,
            created_user_id: false,
            created_time: false,
            created_date: false
        }
    }


    static getTableName(){
        return 'instrument_ip_addresses';
    }

    static async create(instrument_id, instrument_ip_addresses, sessionToken)
    {
        for(let ip_addr of instrument_ip_addresses){
            if(ip_addr.host.length > 0){
                ip_addr.instrument_id = instrument_id;
                ip_addr = CodeUtils.setOnlyDefaultProperties(ip_addr, this.Default);
                CodeUtils.fillFieldsCreatedModified(ip_addr, sessionToken);
                ip_addr = await CodeUtils.insertDB(this.getTableName(), ip_addr);
            }
        }
        return instrument_ip_addresses;
    }

    static async update(instrument_id, instrument_ip_addresses, sessionToken)
    {
        let existing_objects = await CodeUtils.getOne2Many(this.getTableName(), 'instrument_id', instrument_id);
        let allow2left = false;
        let obj2left = [];

        for(let ip_addr of instrument_ip_addresses){
            if(ip_addr.hasOwnProperty('id')){
                ip_addr = CodeUtils.setOnlyDefaultProperties(ip_addr, this.Default);
                CodeUtils.fillFieldsCreatedModified(ip_addr, sessionToken);
                ip_addr = await CodeUtils.updateDB(this.getTableName(), ip_addr);
                for (let exist_obj of existing_objects){
                    if(exist_obj.id == ip_addr.id){
                        allow2left = true;
                    }
                }
                if(allow2left){
                    obj2left.push(ip_addr.id);
                }
            }
            else{
                this.create(instrument_id, [ip_addr], sessionToken);
            }
        }

        for (let exist_obj of existing_objects){
            if(obj2left.indexOf(exist_obj.id) == -1){
                await CodeUtils.delete('id', exist_obj.id, sessionToken.user, this.getTableName());
            }
        }

        return instrument_ip_addresses;
    }

    static async delete(item_id, sessionToken){
        try
        {
            await CodeUtils.delete('instrument_id', item_id, sessionToken.user, this.getTableName());
        }
        catch (exc)
        {
            logger.error('InstrumentsController.delete');
            logger.error(exc);
        }
        return true;
    }
}
