const PermissionUtils = require('../utils/PermissionUtils');

const CodeUtils = require('../utils/CodeUtils');
const GeneralUtils = require('../utils/GeneralUtils');
const i18n = require('../utils/i18n');
const logger = require('../common/Logger');
const DateTimeUtils = require('../common/DateTimeUtils');
const pgAsync = require('../utils/Database');
const SQL = require('pg-async').SQL;
const DevelopmentTypes = require('../common/DevelopmentTypes');
const EventTypes = require('../common/EventTypes');

const _ = require('lodash');

module.exports = class RequestsController {

    // avoid multiple requests to be repeatedly registered
    static async isRequestAlreadyRegistered(request)
    {
        let result = false;
        let sql = '';
        try
        {
            sql = SQL`SELECT * FROM requests  
                WHERE 
                    req_timestamp = ${request.timestamp} AND
                    req_hash = ${request.req_hash}
                LIMIT 1`;

            let data = await pgAsync.rows(
                sql
            );
            result = (data.length > 0);
        }
        catch (exc)
        {
            logger.error('RequestsController.isRequestAlreadyRegistered', sql);
            logger.error(exc);
        }

        return result;
    }

    static async registerRequest(request)
    {
        let result = false;
        let sql = '';
        try
        {
            if(!(await RequestsController.isRequestAlreadyRegistered(request))) {

                let str_request = '';
                if ('file' in request && request.file) {
                    let file = request.file;
                    request.file = '[excluded]';
                    let requestClone =  _.clone(request);
                    request.file = file;
                    str_request = JSON.stringify(requestClone);
                }
                else {
                    str_request = JSON.stringify(request);
                }

                sql = SQL`INSERT INTO requests    
                (
                    request,
                    req_timestamp, 
                    req_hash
                ) VALUES
                (
                    ${str_request},
                    ${request.timestamp},
                    ${request.req_hash}
                )`;

                await pgAsync.rows(
                    sql
                );
                result = true;
            }
        }
        catch (exc)
        {
            logger.error('RequestsController.registerRequest', sql);
            logger.error(exc);
        }

        return result;
    }

};
