const logger = require('../common/Logger');
const pgAsync = require('../utils/Database');
const SQL = require('pg-async').SQL;
const InstrumentsController = require('../controllers/InstrumentsController');
const CodeUtils = require('../utils/CodeUtils');
const DateTimeUtils = require("../common/DateTimeUtils");

module.exports = class RolesInstrumentsController{
    static getTableName(){
        return 'roles_instruments';
    }

    static async create(role_id, role_instruments, user)
    {
        let roles_instruments = [];
        try
        {
            var currentDate = (new Date());
            var currentTime = DateTimeUtils.getTimeLocalString(currentDate);
            for (let instrument of role_instruments) {
                let result = await pgAsync.query(
                    SQL`INSERT INTO roles_instruments
                        (
                            created_time,
                            created_date,
                            created_user_id,
                            instrument_id,
                            is_deleted,
                            modified_user_id,
                            modified_time,
                            modified_date,
                            role_id
                        )
                        VALUES
                        (
                            ${currentTime},
                            ${currentDate},
                            ${user.id},
                            ${instrument.id},
                            false,
                            ${user.id},
                            ${currentTime},
                            ${currentDate},
                            ${role_id}
                        ) RETURNING id`);

                roles_instruments.push({
                    'created_time': currentTime,
                    'created_date': currentDate,
                    'created_user_id': user.id,
                    'instrument_id': instrument.id,
                    'is_deleted': false,
                    'modified_user_id': user.id,
                    'modified_time': currentTime,
                    'modified_date': currentDate,
                    'role_id': role_id,
                    'id': result.rows[0].id
                });
            }

        }
        catch (exc)
        {
            logger.error('RolesInstrumentsController.create');
            logger.error(exc);
        }

        return roles_instruments;
    }

    static async update(role_id, role_instruments, user)
    {
        let existing_instruments = await CodeUtils.getForMany2Many(role_id, 'roles_instruments', 'role_id', 'instruments', 'instrument_id');
        await CodeUtils.updateMany2Many(role_id, role_instruments, user, existing_instruments, this.delete, this.create);
    }

    static async delete(role_id, instrument_id, user){
        let options = {
            sub_item_name: 'instrument_id',
            sub_item_id: instrument_id
        };
        await CodeUtils.delete('role_id', role_id, user, 'roles_instruments', options);
    }

    static async isInstrumentUsing(id){
        try
        {
            var is_uniq = await CodeUtils.isUniqFieldData(this.getTableName(), 'instrument_id', id);
        }
        catch (exc)
        {
            logger.error('RolesInstrumentsController.isInstrumentUsing');
            logger.error(exc);
        }

        return is_uniq;
    }
}
