const logger = require('../common/Logger');
const pgAsync = require('../utils/Database');
const SQL = require('pg-async').SQL;
const ProjectGroupsController = require('../controllers/ProjectGroupsController');
const CodeUtils = require('../utils/CodeUtils');
const DateTimeUtils = require("../common/DateTimeUtils");

module.exports = class RolesProjectGroupsController{
    static async create(role_id, project_groups, user)
    {
        let roles_project_groups = [];
        try
        {
            var currentDate = (new Date());
            var currentTime = DateTimeUtils.getTimeLocalString(currentDate);
            for (let p_groups of project_groups) {
                let result = await pgAsync.query(
                    SQL`INSERT INTO roles_project_groups
                        (
                            created_time,
                            created_date,
                            created_user_id,
                            is_deleted,
                            modified_user_id,
                            modified_time,
                            modified_date,
                            project_group_id,
                            role_id
                        )
                        VALUES
                        (
                            ${currentTime},
                            ${currentDate},
                            ${user.id},
                            false,
                            ${user.id},
                            ${currentTime},
                            ${currentDate},
                            ${p_groups.id},
                            ${role_id}
                        ) RETURNING id`);

                roles_project_groups.push({
                    'created_time': currentTime,
                    'created_date': currentDate,
                    'created_user_id': user.id,
                    'is_deleted': false,
                    'modified_user_id': user.id,
                    'modified_time': currentTime,
                    'modified_date': currentDate,
                    'project_group_id': p_groups.id,
                    'role_id': role_id,
                    'id': result.rows[0].id
                });
            }

        }
        catch (exc)
        {
            logger.error('RolesProjectGroupsController.create');
            logger.error(exc);
        }

        return roles_project_groups;
    }

    static async update(role_id, project_groups, user)
    {
        let existing_project_groups = await CodeUtils.getForMany2Many(role_id, 'roles_project_groups', 'role_id', 'project_groups', 'project_group_id');
        await CodeUtils.updateMany2Many(role_id, project_groups, user, existing_project_groups, this.delete, this.create);
    }

    static async delete(role_id, project_group_id, user){
        let options = {
            sub_item_name: 'project_group_id',
            sub_item_id: project_group_id
        };
        await CodeUtils.delete('role_id', role_id, user, 'roles_project_groups', options);
    }
}
