const logger = require('../common/Logger');
const pgAsync = require('../utils/Database');
const SQL = require('pg-async').SQL;
const CodeUtils = require('../utils/CodeUtils');

module.exports = class UsersRolesController {
    static get Default() {
        return {
            id: 0,
            role_id: 0,
            user_id: 0,

            is_deleted: false,
            modified_user_id: false,
            modified_time: false,
            modified_date: false,
            created_user_id: false,
            created_time: false,
            created_date: false,
        }
    }

    static getTableName(){
        return 'users_roles';
    }

    static async isRoleUsing(role_id){
        try
        {
            var is_uniq = await CodeUtils.isUniqFieldData(this.getTableName(), 'role_id', role_id);
        }
        catch (exc)
        {
            logger.error('UsersRolesController.isRoleUsing');
            logger.error(exc);
        }

        return is_uniq;
    }

    static async create(user_id, user_roles, user)
    {
        return await CodeUtils.createMany2Many('user_id', user_id, 'role_id', user_roles, 'users_roles', user)
    }

    static async update(user_id, user_roles, user)
    {
        let existing_roles = await CodeUtils.getForMany2Many(user_id, 'users_roles', 'user_id', 'roles', 'role_id');
        await CodeUtils.updateMany2Many(user_id, user_roles, user, existing_roles, this.delete, this.create);

    }

    static async delete(user_id, role_id, user)
    {
        let options = {
            sub_item_name: 'role_id',
            sub_item_id: role_id
        };

        await CodeUtils.delete('user_id', user_id, user, 'users_roles', options);
        return true;
    }

}
