var _ = require('lodash');

module.exports = class PermissionUtils {

    static checkAssocPermission(items_array, item_type, sessionToken){
        let can_access_name = '';
        let filterProjects = false;
        switch (item_type) {
            case 'instruments':
                can_access_name = 'can_access_all_instruments';
                break;
            case 'reports':
            case 'projects':
                can_access_name = 'can_access_all_project_groups';
                item_type = 'projectGroups';
                filterProjects = true;
                break;
            case 'projectGroups':
                can_access_name = 'can_access_all_project_groups';
                break;
        }

        let merged_can_access = sessionToken.user[can_access_name];
        for(let role_can_access of sessionToken.user.roles){
            merged_can_access = merged_can_access || role_can_access[can_access_name];
        }

        if(merged_can_access){
            return items_array;
        }

        let merged_allowed_item_id = []
        for(let allowed_user_item of sessionToken.user[item_type]){
            if(merged_allowed_item_id.indexOf(allowed_user_item.id) == -1){
                merged_allowed_item_id.push(allowed_user_item.id);
            }
        }
        for(let role of sessionToken.user.roles){
            for(let role_allowed_item of role[item_type]){
                if(merged_allowed_item_id.indexOf(role_allowed_item.id) == -1){
                    merged_allowed_item_id.push(role_allowed_item.id);
                }

            }
        }

        if(_.isArray(items_array)){
            if(filterProjects){
                _.remove(items_array, function(item){
                    if(item.project_group_id === undefined){
                        return merged_allowed_item_id.indexOf(item.project.project_group_id) == -1;
                    }
                    else{
                        return merged_allowed_item_id.indexOf(item.project_group_id) == -1;
                    }

                });
            }
            else{
                _.remove(items_array, function(item){
                    return merged_allowed_item_id.indexOf(item.id) == -1;
                });
            }
        }
        else{
            if(filterProjects){
                if(items_array.project_group_id === undefined){
                    if(merged_allowed_item_id.indexOf(items_array.project.project_group_id) == -1){
                        items_array = false;
                    }
                }
                else{
                    if(merged_allowed_item_id.indexOf(items_array.project_group_id) == -1){
                        items_array = false;
                    }
                }
            }
            else{
                if(merged_allowed_item_id.indexOf(items_array.id) == -1){
                    items_array = false;
                }
            }
        }

        return items_array;
    }

    static joinAccessProps(sessionToken){
        sessionToken.joint_can_manage_users = sessionToken.user.can_manage_users;
        sessionToken.joint_can_access_admin_console = sessionToken.user.can_access_admin_console;
        sessionToken.joint_can_access_all_instruments = sessionToken.user.can_access_all_instruments;
        sessionToken.joint_can_access_all_project_groups = sessionToken.user.can_access_all_project_groups;
        sessionToken.allowed_project_group_ids = [];
        sessionToken.allowed_instrument_ids = [];
        for(let instrument of sessionToken.user.instruments){
            sessionToken.allowed_instrument_ids.push(instrument.id);
        }
        for(let project_group of sessionToken.user.projectGroups){
            sessionToken.allowed_project_group_ids.push(project_group.id);
        }
        for(let role of sessionToken.user.roles){
            sessionToken.joint_can_manage_users = sessionToken.joint_can_manage_users || role.can_manage_users;
            sessionToken.joint_can_access_admin_console = sessionToken.joint_can_access_admin_console || role.can_access_admin_console;
            sessionToken.joint_can_access_all_instruments = sessionToken.joint_can_access_all_instruments || role.can_access_all_instruments;
            sessionToken.joint_can_access_all_project_groups = sessionToken.joint_can_access_all_project_groups || role.can_access_all_project_groups;

            for(let pg of role.projectGroups){
                sessionToken.allowed_project_group_ids.push(pg.id);
            }

            for(let instr of role.instruments){
                sessionToken.allowed_instrument_ids.push(instr.id);
            }
        }
        return sessionToken;
    }

    static getAllowedIds(sessionToken, type, field_name, options){

        let _options = _.defaults(options || {}, {
            allow_zero_ids: false,
            allow_null_ids: false
        });

        let permissionFilter = '';
        let ids = '';

        if(sessionToken === undefined){
            return permissionFilter;
        }

        if(type == 'instruments'){
            if(!sessionToken.joint_can_access_all_instruments){
                if(sessionToken.allowed_instrument_ids.length > 0){
                    ids = sessionToken.allowed_instrument_ids.join();
                    if(_options.allow_zero_ids){
                        ids += ',0';
                    }
                }
                else{
                    ids = '-1';
                }
                permissionFilter = ' AND '+field_name+' IN (' + ids + ') ';
            }
        }

        if(type == 'projectGroups'){
            if(!sessionToken.joint_can_access_all_project_groups){
                if(sessionToken.allowed_project_group_ids.length > 0){
                    ids = sessionToken.allowed_project_group_ids.join();
                    if(_options.allow_zero_ids){
                        ids += ',0';
                    }
                }
                else{
                    ids = '-1';
                }
                if(_options.allow_null_ids){
                    permissionFilter = ' AND ('+field_name+' IN (' + ids + ') OR '+field_name+' IS NULL) ';
                }
                else{
                    permissionFilter = ' AND '+field_name+' IN (' + ids + ') ';
                }
            }
        }
        return permissionFilter;
    }
}
